/*
 * This file provides definitions for the "curve" data. 
 */
 
namespace java com.truecar.thrift.pricing
namespace cocoa TCThrift

include "common/Numeric.thrift"
include "common/Monetary.thrift"


/**
 * This enumeration indicates the reasons why pricing data is not available
 * in those cases where it is missing.  The basic reasons are:
 *
 * <dl>
 * <dt>NEW</dt>
 * <dd>The vehicle trim has just been released, and pricing data is not yet
 * available.
 * </dd>
 * <dt>OLD</dt>
 * <dd>At the end of the model year, the vehicle trim is no longer active.
 * </dd>
 * <dt>NO_DATA</dt>
 * <dd>No data is available for this trim.
 * </dd>
 * </dl>
 */
enum PricingAbsenceReason {
	NEW = 0,
	OLD = 1,
	NO_DATA = 2
}


/**
 * This represents a single pricing data point.  Pricing structures generally
 * support one of two basic models:
 *
 * <dl>
 * <dt>actual</dt>
 * <dd>Actual prices represent an absolute value.  These are used as a general
 * case for consumer pricing information.
 * </dd>
 * <dt>relative</dt>
 * <dd>In cases of relative pricing, offsets from some fixed value are
 * represented. These offsets may appear as positive or negative values.  This
 * form is used for dealer curve data, where data generally appears as offsets
 * from the factory invoice price.
 * </dd> 
 * </dl>
 */
struct Price {
	/**
	 * This is an actual price value - i.e. an absolute amount.  These values
	 * are always positive.
	 */
	1:optional Monetary.UsdCents amount,
	
	/**
	 * This is an offset from some known value.  The offset may be positive
	 * or negative.
	 */
	2:optional Monetary.UsdCents offset
}


/**
 * This structure represents a single plotted point on a consumer curve.
 */
struct PricePoint {
	/**
	 * This is the actual price value - either an actual value of an offset,
	 * depending on the usage. 
	 */
	1:required Price price,
	
	/**
	 * This is the position along the curve presented as a percentage: a value
	 * between 0.0 and 1.0.
	 */ 
	2:optional Numeric.Percentage percent,
	
	/** This is the price type.  It is only relevant for callout prices. */
	3:optional string type,
	
	/** This is a description.  It is only relevant for callout prices. */
	4:optional string desc,	
}


/**
 * This structure represents a collection of plotted points on a curve.  It is
 * considered an invariant that all <code>PricePoint</code> instances contain
 * <code>Price</code> instances of the same type.  That is, prices are
 * consistently represented as actual or offset values.
 * <p>
 * The structure assumes that the pricing information is partitioned into four
 * regions for presentation purposes:
 * </p>
 *
 * <ol>
 * <li>Below market prices.</li>
 * <li>"Great" prices.</li>
 * <li>"Good" prices.</li>
 * <li>Above market prices.</li>
 * </ol>
 *
 * <p>
 * The four regions are represented by three price points, each of which
 * identifies the minimum value for a given region.
 * </p>
 */
struct PricePoints {
	/** The average price. */
	1: optional PricePoint average,
	
	/** The invoice price. */
	2: optional PricePoint invoice,
	
	/** The manufacturer's suggested retail price. */
	3: optional PricePoint msrp,
	
	/** The callout price. */
	4: optional PricePoint callout,
	
	/** The dealer cost. */
	5: optional PricePoint dealerCost,
	
	/**
	 * The minimum value for a "good" price.  This is the least value for the
	 * good price region; values immediately below this are part of the "great"
	 * price region.
	 */
	6: optional PricePoint goodMinimum,
	
	/**
	 * The minimum value for a "great" price.  This is the least value for the
	 * great price region; values immediately below this are part of the "below
	 * market" region.
	 */
	7: optional PricePoint greatMinimum,
	
	/**
	 * The minimum value for an above-market price.  This is the least value
	 * for the above-market region; values immediately below this are part of
	 * the good price region.
	 */
	8: optional PricePoint aboveMarketMinimum,
}


/**
 * This provides the data for consumer and/or dealer curves.  For consumer
 * curves, the encapsulated <code>Price</code> instances will hold
 * actual values.  For dealer curves, the price data will hold offsets. 
 */
struct Curve {
	/** The set of price points for the curve. */
	1: optional PricePoints prices,
	
	/** The number of industry transactions represented in the calculations. */
	2: optional i32 industryTransactions,
	
	/** The number of TCDC transactions represented in the calculations. */
	3: optional i32 tcdcTransactions,
	
	/**
	 * This is the statistical certainty of the pricing information based on
	 * sample size, recency, and accuracy.  
	 */
	4: optional Numeric.Percentage confidence,
	
	/**
	 * This value denotes the margin of variability between average paid and
	 * market average.  That is, TrueCar is 'confidence' confident
	 * that the average paid is within 'confidenceWithin' percent
	 * of the market average.  
	 */
	5: optional Numeric.Percentage confidenceWithin,
	
	/**
	 * If present, this field provides the rationale as to why pricing
	 * information is missing.
	 */
	6: optional PricingAbsenceReason specialCase
}


/**
 * This structure represents a single bucket for historgram data.  Note that
 * customer histograms use <code>Price</code> instances with actual values,
 * whereas dealer histograms are composed of offset values.  As an invariant,
 * instances of <code>Price</code> are homogeneous - that is, all are
 * consistently either actual or offset values.
 */
struct Bucket {
	/** The number of transactions represented in the bucket. */
	1: required i32 transactionCount,
	
	/** The minimum value. */
	2: required Price minimum,
	
	/** The maximum value. */
	3: required Price maximum,
	
	/**
	 * The minimum relative percentage.  This is calculated as:
	 *
	 * <code>
	 *   (minimumAmount - msrp) / msrp
	 * </code>
	 *
	 * Values generally like in the range of -1.0 and +1.0, scaled using the
	 * standard representation.
	 */
	4: optional Numeric.Percentage minimumMsrpRelative,

	/**
 	 * The maximum relative percentage.  This is calculated as:
 	 *
 	 * <code>
 	 *   (maximumAmount - msrp) / msrp
 	 * </code>
	 *
	 * Values generally like in the range of -1.0 and +1.0, scaled using the
	 * standard representation.
 	 */
	5: optional Numeric.Percentage maximumMsrpRelative,	
}


/**
 * This is the collection of data for a single histogram chart.  As an
 * invariant, all contained <code>Bucket</code> instances are homogeneous
 * - that is, all consistently contain <code>Price</code> values that
 * are actual or offset values. 
 */
struct Histogram {
	/** The number of days encompassed in the histogram data. */
	1: required i16 historicalDays,
	
	/* The total transaction count encompassed in the histogram. */
	2: required i32 totalTransactionCount,
	
	/** The histogram data. */
	3: required list<Bucket> buckets,
}


/**
 * This provides a grouping of chart data elements that are tied to a specific
 * locality.  Currently, there are only two chart types: curves, and histograms.
 */
struct ChartSet {
	/**
	 * The name of the locality.  Naming varies with type:
	 *
	 * <dl>
	 * <dt>local</dt>
	 * <dd>The name takes the form: "City, ST".</dd>
	 * <dt>regional</dt>
	 * <dd>The name is a "State".</dd>
	 * <dt>national</dt>
	 * <dd>The field is omitted.</dd>
	 * </dl>
	 */
	1: optional string localityName,
	
	/**
	 * The curve data.  This field may be omitted if not requested via the API,
	 * or if it is not available for the given locality.
	 */
	2: optional Curve curve,
	
	/**
	 * The histogram data.  As with the curve, this field may be omitted if not
	 * requests via the API, or if it is not available for the given locality.
	 */
	3: optional Histogram histogram,	
}


/**
 * A set of charts for all localities.
 */
struct Charts {
	/** National charts. */
	1: optional ChartSet nationalCharts,
	
	/** Regional charts. */
	2: optional ChartSet regionalCharts,
	
	/** Local charts. */
	3: optional ChartSet localCharts,
}
