namespace cocoa TCThrift
namespace java com.truecar.mobile.thriftApi.pricecheck

include "common/Geo.thrift"
include "common/Monetary.thrift"
include "common/Numeric.thrift"
include "common/Temporal.thrift"
include "common/DealerFeeOrAccessory.thrift"
include "common/PriceEmphasis.thrift"

struct PriceCheckVin
{
    1:required string vin,                          //This is a candidate VIN. Note that it is possible that this may not be a full VIN.
    2:required Numeric.Percentage confidence,       //This is a percentage value in the range from 0.0 to 1.0. This represents the image service's confidence in the quality of the VIN decoding for this particular value.
}


//  --------------------------------------------------------------------------------------------------------------------
//
//	Translate Image To Vin API Endpoint
// 
//	URL: {baseUrl}/pricecheck/pag/{pag}/image/windowvin
//
//	Request:
//		POST {baseUrl}/pricecheck/pag/{pag}/image/windowvin HTTP/1.1
//		Host: {host}
//		Content-Type: application/octet-stream
//		Content-Length: 0
//		Accept: application/octet-stream
//
//	Response (success):
//		HTTP/1.1 200 OK
//		Content-Type: application/x-thrift
//		Content-Length: 0
//
//	Response (error):
//		HTTP/1.1 {non200Status}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{Exception}
//
//	Potential Error Codes:
//		UhOh
//
//  --------------------------------------------------------------------------------------------------------------------
struct TranslateImageToVinRequest
{
    1:required string mimeType,
    2:required binary data,
}

struct TranslateImageToVinResponse
{
    1:required i64 jobId,
    2:required i64 expectedCompletionMillis,
    3:required list<PriceCheckVin> vins
}

/**
 *	Code to indicate reason for failure to determine price.
 */
enum PriceCheckReason
{
	/**
	 *	Price was successfully determined.
	 */
	SUCCESS,
	
	/**
	 *	Location required to calculate price.
	 */
	NEED_LOCATION,
	
	/**
	 *	PAG ID required to calculate price.
	 */
	NEED_PAG_ID,
	
	/**
	 *	Dealer ID required to calculate price.
	 */
	NEED_DEALER_ID,
	
	/**
	 *	VINs required to calculate price.
	 */
	NEED_VINS,
	
	/**
	 *	MSRP required to calculate price.
	 */
	NEED_MSRP,
	
	/**
	 *	Make ID required to calculate price.
	 */
	NEED_MAKE_ID,
	
	/**
	 *	Year required to calculate price.
	 */
	NEED_YEAR,
	
	/**
	 *	Model ID required to calculate price.
	 */
	NEED_MODEL_ID,
	
	/**
	 *	Trim ID required to calculate price.
	 */
	NEED_TRIM_ID,
	
	/**
	 *	Package IDs required to calculate price.
	 */
	NEED_PACKAGE_IDS,
	
	/**
	 *	Option IDs required to calculate price.
	 */
	NEED_OPTION_IDS,
	
	/**
	 *	Incentive IDs required to calculate price.
	 */
	NEED_INCENTIVE_IDS,
	
	/**
	 *	Total of dealer installed options required to calculate price.
	 */
	NEED_DEALER_INSTALLED_OPTIONS_TOTAL,
    
    /**
     * User must give content or decline to contact the dealer.
     */
    NEED_CONSENT_TO_CONTACT_DEALER,
}

/**
 *	Code to indicate how to obtain needed value.
 */
enum PriceCheckHint
{
	/**
	 *	No hint.
	 */
	NONE,
	
	/**
	 *	Allow user to enter VIN by last 5 characters only.
	 */
	VINS_ALLOW_5_CHAR_MATCH,
	
	/**
	 *	Response contains list of possible VINs to pick from.
	 *
	 *	This hint may be used both during manual entry as a search results or if the results of an OCR are ambiguous.
	 */
	VINS_PICK_FROM_POSSIBLE_VALUES,
    
    /**
     * success response but the incentives were picked from defaults, not from the user.
     */
    SUCCESS_INCENTIVES_IN_POSSIBLE_VALUES,
}

enum PriceCheckConsentToContactDealer
{
	/**
	 *	User has not been asked for consent. i.e. The user has not seen the Welcome screen yet.
	 */
    NOT_YET_ASKED,
    
    /**
	 *	User has denied consent for the application to notify the dealer that the user is on the lot.
     */
    DENIED,
    
    /**
	 *	User has granted consent for the applicaiton to notify the dealer that the user is on the lot.
	 */
    GRANTED
}

/**
 *	Contact information for one person at dealer.
 */
struct PriceCheckResultDealerContact
{
	/**
	 *	Name.
	 */
    1:optional string name,

	/**
	 *	Title.
	 */
    2:optional string title,

	/**
	 *	E-mail address.
	 */
    3:optional string email,

	/**
	 *	Phone number.
	 */
	4:optional string phone,

	/**
	 *	Quotation.
	 */
    5:optional string quote,

	/**
	 *	URL of image.
	 */
    6:optional string imageUrl,
    
    /**
     * Whether or not this contact is the individual who will be contact as a result of the PriceCheck action.
     */
    7:optional bool priceCheckContact,
    
    /**
     * The ID of the "Dealer" (not the Dealership).
     */
    8:optional i64 dealerContactId, 
}

/**
 *	Dealer information for Price Check result.
 */
struct PriceCheckResultDealer
{
	/**
	 *	Dealer ID.
	 */
	1:required i64 id,

	/**
	 *	Name.
	 */
	2:optional string name,

	/**
	 *	Street address, line 1.
	 */
	3:optional string streetAddress1,

	/**
	 *	Street address, line 2.
	 */
	4:optional string streetAddress2,

	/**
	 *	City.
	 */
	5:optional string city,

	/**
	 *	State.
	 */
	6:optional string state,

	/**
	 *	Postal code.
	 */
	7:optional string postalCode,

	/**
	 *	Phone number.
	 */
	8:optional string phone,

	/**
	 *	Location.
	 */
	9:optional Geo.GeoPoint location,

	/**
	 *	Hours of operation.
	 */
	10:optional Temporal.HoursOfOperation hours,

	/**
	 *	Contacts.
	 *
	 *	First contact in list is primary contact. Second contact in list is secondary contact. Primary and secondary
	 *	contact are directly accessible for email and phone in the price check UI, without opening the dealer profile.
	 */
	11:optional list<PriceCheckResultDealerContact> contacts,

    /**
     * Copy of the imageUrl attribute from the VehicleDealPhoto.
     */
    12:optional list<string> imageUrls,
    
    /**
     * Copy of the youTubeId attribute from the VehicleDealVideo.
     */
    13:optional list<string> youTubeIds,
    
    /**
     * Flag to indicate that this dealerhip is usable for PriceCheck.
     */
    14:optional bool priceCheckEnabled,
}

/**
 *	Option in Price Check result.
 */
struct PriceCheckResultOption {
	/**
	 *	Option ID.
	 *
	 *	The server may choose to use a trim-specific option ID here as long as there is consistency with the IDs used in
	 *	the request and the possible values of a response.
	 */
	1:required i64 id,
	
	/**
	 *	Option name.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
	2:optional string name,
	
	/**
	 *	Option description.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
	3:optional string desc,
	
	/**
	 *	Option MSRP.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
	4:optional Monetary.UsdCents msrp,
	
	/**
	 *	Option category name.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
	5:optional string category,
}

/**
 *	Incentive in Price Check result.
 */
struct PriceCheckResultIncentive {
	/**
	 *	Incentive ID.
	 */
	1:required i64 id,
	
	/**
	 *	Incentive name.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
	2:optional string name,
	
	/**
	 *	Incentive description.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
	3:optional string desc,

	/**
	 *	Incentive start date.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
    4:optional Temporal.DateIsoString startDate,

	/**
	 *	Incentive end date.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
    5:optional Temporal.DateIsoString endDate,

	/**
	 *	Incentive amount.
	 *
	 *	May be omitted in intermediate responses, but should be set in final Price Check result.
	 */
	6:optional Monetary.UsdCents amount,
    
    /**
     * This value is set to true when the incentive type is 2 (Dealer) or incentive is preselected.
     */
    7:optional bool isIncludedInGuaranteedSavings,
}

/**
 *	Result of Price Check.
 *
 *	When successful, this will be completely filled out with the final price of the vehicle. Prior to obtaining a final
 *	price, this may contain some elements to indicate decisions made by the server. For example, a request containing
 *	multiple possible VINs may not result in a final price, but the server will resolve the set down to a single VIN to
 *	be used in subsequent requests. That single vin will be returned in this result.
 */
struct PriceCheckResult
{
	/**
	 *	Vehicle identification number.
	 */
	1:optional string vin,
	
	/**
	 *	Make ID.
	 */
	2:optional i64 makeId,
	
	/**
	 *	Make name.
	 */
	3:optional string makeName,
	
	/**
	 *	Year.
	 */
	4:optional i16 year,
	
	/**
	 *	Model ID.
	 */
	5:optional i64 modelId,
	
	/**
	 *	Model name.
	 */
	6:optional string modelName,
	
	/**
	 *	Trim ID.
	 */
	7:optional i64 trimId,
	
	/**
	 *	Trim name.
	 */
	8:optional string trimName,
	
	/**
	 *	Total price before fees.
	 */
	9:optional Monetary.UsdCents priceBeforeFeesOrAccessories,
	
	/**
	 *	Savings off MSRP, excluding incentives.
	 */
	10:optional Monetary.UsdCents savings,
	
	/**
	 *	Savings percentage off MSRP.
	 */
	11:optional Numeric.Percentage savingsPercent,
	
	/**
	 *	MSRP.
	 */
	12:optional Monetary.UsdCents msrpTotal,
	
	/**
	 *	Base price.
	 */
	13:optional Monetary.UsdCents baseMsrpAmount,
	
	/**
	 *	Total price of dealer installed options.
	 */
	14:optional Monetary.UsdCents dealerInstalledOptionsTotal,

	/**
	 *	Destination fee.
	 */
	15:optional Monetary.UsdCents destinationFee,

	/**
	 *	Documentation fee.
	 */
	16:optional Monetary.UsdCents deprecatedDocumentationFee,

	/**
	 *	Total MSRP of options.
	 */
	17:optional Monetary.UsdCents optionsMsrpTotal,

	/**
	 *	Total amount of all incentives that are included in the guaranteed savings.
	 */
	18:optional Monetary.UsdCents additionalIncentivesTotal,

	/**
	 *	Savings off MSRP, including incentives EXCEPT for the ones that are NOT included in guaranteed savings.
	 */
	19:optional Monetary.UsdCents guaranteedSavings,

	/**
	 *	Complete dealer information.
	 */
	20:optional PriceCheckResultDealer dealer,
	
	/**
	 *	Complete options information.
	 */
	21:optional list<PriceCheckResultOption> options,

	/**
	 *	Complete incentives information.
	 */
	22:optional list<PriceCheckResultIncentive> incentives,
    
    /**
     * This is priceTotal plus anything else added by the dealer/user to the price.
     */
    //DEPRECATED: This field is not used anymore: 23:optional Monetary.UsdCents priceGrandTotal,
    
   	/**
	 * This is (savings - guaranteedSavings - additionalIncentivesTotal)
	 */
	//DEPRECATED: This field is not used anymore: 24:optional Monetary.UsdCents optionsSavings,
    
    /**
	 *  On Sept. 6, 2013, this meant: Base MSRP + Total Options MSRP + Destination Fee + Dealer Installed Options
     *  On Aug. 11, 2014, this field was deprecated and meant: (priceBeforeFees + dealerInstalledOptionsTotal)
	 */
	25:optional Monetary.UsdCents deprecatedTotalRetailPrice,
    
    26:optional string deprecatedDocumentationFeeName,
    
    27:optional Monetary.UsdCents priceTotalPlusDocumentationFee,
    
    28:optional Temporal.DateIsoString expirationDate,
    
    29: optional Monetary.UsdCents deprecatedMaxStateDocumentationFee,
    
    30: optional string interiorColorName,
    
    31: optional string exteriorColorName,
    
    /**
     * Deprecated in MSP 6.0.0+.  Use "specialFeesInvoiceTotal" instead.
     */
    32: optional Monetary.UsdCents deprecatedRegionalFeesPrice,

    33: optional Monetary.UsdCents estimatedTaxPrice,
    
    34: optional Monetary.UsdCents estimatedTitlePrice,
    
    35: optional Monetary.UsdCents estimatedLicensePrice,
    
    36: optional Monetary.UsdCents estimatedRegistrationPrice,

    37: optional string imageUrl,
    
    /**
     *	List of dealer fees.
     */
    38: optional list<DealerFeeOrAccessory.DealerFeeOrAccessory> dealerFees,
    
	/**
     * Dealer transparency pledge.
     */
    39: optional bool dealerTransparencyPledge,   
    
    /**
     * This value indicates the price to emphasize on. e.g. price, price + dealer installed options + doc fee etc.
     *
     */
    40: optional PriceEmphasis.PriceEmphasis priceEmphasis,
    
	/**
	 *	Total price with fees and accessories
	 */
	41: optional Monetary.UsdCents priceWithFeesAndAccessories,   
	
	/**
	 * Fees total.
	 */
	42: optional Monetary.UsdCents totalFeesAndAccessories,
    
	/**
     * Whether or not the transparency pledge badge should be shown (note this is different than the dealerTransparencyPledge flag).
     */
    43: optional bool showDealerTransparencyPledge,
    
    /**
     * Regional fees value (should be specialFeesMsrpTotal from platform APIs)
     */
    44: optional Monetary.UsdCents specialFeesMsrpTotal,
}

/**
 *	Possible value rule.
 *
 *	A single rule attached to a possible value. Primarily for options, this can indicate when an option requires other
 *	options, includes other options, or excludes other options.
 */
struct PriceCheckPossibleValueRule
{
	/**
	 *	Operator.
	 *
	 *	R - requires other values identified in operands.
	 *  I - includes other values identified in operands.
	 *  X - excludes other values identified in operands.
	 *  O - or-requires other values identified in operands.
	 *  N - or-includes other values identified in operands.
	 *	M - mutally requires other values identified in operands.
	 *  H - hides other values identified in operands.
	 *
	 *	The app may choose to implement only a subset of the possible operators.
	 */
	1:required string op,
	
	/**
	 *	Operands.
	 *
	 *	The operands identify the other values affected by the rule.
	 */
	2:required list<i64> operands,
}

/**
 *	Possible value.
 *
 *	For steps that require the user to choose from a list, this provides the data for one element in the list.
 */
struct PriceCheckPossibleValue
{
	/**
	 *	Identifier for item.
	 */
	1:optional i64 id,
	
	/**
	 *	Name of item.
	 */
	2:optional string name,
	
	/**
	 *	Description for item.
	 */
	3:optional string desc,
	
	/**
	 *	Price of item.
	 */
	4:optional Monetary.UsdCents price,
	
	/**
	 *	URL of image for item.
	 */
	5:optional string imageUrl,
	
	/**
	 *	Flag to indicate if item should be pre-selected.
	 */
	6:optional bool preselected,
	
	/**
	 *	Rules for item.
	 */
	7:optional list<PriceCheckPossibleValueRule> rules,
    
    /**
     * Flag to indicate that this option is a radio selection and should be exclusive within the group.
     */
	8:optional bool radioSelection,
}

/**
 *	Possible value group.
 *
 *	For steps that require the user to choose from a list, this provides a grouping of multiple items. Primarily used
 *	for options where a group corresponds to an option category. When no grouping is to be performed, a single group
 *	without a name should be used to contain all the items.
 */
struct PriceCheckPossibleValueGroup
{
	/**
	 *	Name for group.
	 */
	1:optional string name,
	
	/**
	 *	Possible values.
	 */
	2:optional list<PriceCheckPossibleValue> possibleValues,
}

/**
 *	Request for Price Check API Endpoint
 *
 *	URL: {baseUrl}/pricecheck/pag/{pag}/check
 *
 *	Request:
 *		POST {baseUrl}/pricecheck/pag/{pag}/check HTTP/1.1
 *		Host: {host}
 *		Accept: application/x-thrift
 *
 *		{PriceCheckRequest}
 *
 *	Response (success):
 *		HTTP/1.1 200 OK
 *		Content-Type: application/x-thrift
 *		Content-Length: {length}
 *
 *		{PriceCheckResponse}
 *
 *	Response (error):
 *		HTTP/1.1 {non200Status}
 *		Content-Type: application/x-thrift
 *		Content-Length: {length}
 *
 *		{Exception}
 *
 *  Potential Error Codes:
 *		TBD
 */
struct PriceCheckRequest
{
	/**
	 *	Location of user.
	 *
	 *	Use of Price Check requires location obtained from device at the time Price Check was started and will be used
	 *	to verify that user is at a TrueCar dealer (within some reasonable margin).
	 */
	1:optional Geo.GeoPoint location,
	
	/**
	 *	Program affinity group ID.
	 *
	 *	This is the PAG ID used for calculating the price and does not necessarily match the PAG ID provided in the URL.
	 *	For example, the Price Check may be done with the TrueCar app (PAG ID in URL = 3) based on a partner-branded
	 *  certificate. In that case, this PAG ID is for the partner, and may indicate that a partner-specific price
	 *	adjustment can be used in the calculation.
	 */
	2:optional i64 pagId,

	/**
	 *	Dealer ID.
	 *
	 *	When dealer ID is provided, the location of the identified dealer match the location of the user. If not, the
	 *	response will indicate that the dealer ID is still needed.
	 */
	3:optional i64 dealerId,
	
	/**
	 *	Vehicle identification numbers.
	 *
	 *	This is a list of possible VINs. When provided with a dealer ID, the server may choose the best match from the
	 *	dealer's inventory. If manually entered or retrieved via a barcode, this list will only have one item. When
	 *  retrieved via OCR (see Translate Image to Vin API endpoint), this list may often have multiple items.
	 *
	 *	Because OCR is inexact, the vins provided may consist of multiple words and be much longer or shorter than an
	 *	actual VIN. The server should attempt to extract a valid VIN from the provided value, guided by its own
	 *	knowledge of VIN structure, inventory, and VIN decoding.
	 */
	4:optional list<PriceCheckVin> vins,
	
	/**
	 *	Manufacturer's suggested retail price.
	 *
	 *	Total price of vehicle as shown on sticker and entered by user.
	 */
	5:optional Monetary.UsdCents msrp,
	
	/**
	 *	Make ID.
	 */
	6:optional i64 makeId,
	
	/**
	 *	Model year.
	 */
	7:optional i16 year,
	
	/**
	 *	Model ID.
	 */
	8:optional i64 modelId,
	
	/**
	 *	Trim ID.
	 */
	9:optional i64 trimId,
	
	/**
	 *	Package IDs.
	 *
	 *	This is a list of the package IDs as shown on the sticker and selected by the user.
	 */
	10:optional list<i64> packageIds,
	
	/**
	 *	Option IDs.
	 *
	 *	This is a list of the option IDs as shown on the sticker and selected by the user. The server may choose to use
	 *	trim-specific option IDs here as long as there is consistency with the IDs provided in the possible values of a
	 *	response.
	 *
	 *	Also, there is no distinction in this API between having the user select from a full set of options or a limited
	 *	set of options. This is controlled by the server via the possible values of a response.
	 */
	11:optional list<i64> optionIds,
	
	/**
	 *	Incentive IDs.
	 */
	12:optional list<i64> incentiveIds,
	
	/**
	 *	Total price of dealer installed options.
	 */
	13:optional Monetary.UsdCents dealerInstalledOptionsTotal,
    
    /** 
     * DEPRECATED AT 3.4
     *
     * When set to false the server should behave as if 'consentToContactDealer' is set to NOT_YET_ASKED.
     * When set to true the server should behave as if 'consentToContactDealer' is set to DENIED because the new 
     * requirement is that the user must explicitly grant consent and <3.4 clients do not ask this question.
     */
    14:optional bool deprecatedConsentGranted,
    
    /**
     * If this value is set it should be used in preference to the deprecated 'consentGranted' flag.
     * If neither 'consentGranted' nor 'consentToContactDealer' are set the server should never return a reason 
     * of NEED_CONSENT_TO_CONTACT_DEALER because this is an old client that doesn't understand the welcome screen.
     *
     * When set to NOT_YET_ASKED the server may return a reason of NEED_CONSENT_TO_CONTACT_DEALER.
     * When set to DENIED the server may proceed to the next step but must not notify the dealer.
     * When set to GRANTED the server may proceed to the next step and notify the dealer.
     */
    15:optional PriceCheckConsentToContactDealer consentToContactDealer
}

/**
 *	Response for Price Check API Endpoint
 *
 *	URL: {baseUrl}/pricecheck/pag/{pag}/check
 *
 *	Request:
 *		POST {baseUrl}/pricecheck/pag/{pag}/check HTTP/1.1
 *		Host: {host}
 *		Accept: application/x-thrift
 *
 *		{PriceCheckRequest}
 *
 *	Response (success):
 *		HTTP/1.1 200 OK
 *		Content-Type: application/x-thrift
 *		Content-Length: {length}
 *
 *		{PriceCheckResponse}
 *
 *	Response (error):
 *		HTTP/1.1 {non200Status}
 *		Content-Type: application/x-thrift
 *		Content-Length: {length}
 *
 *		{Exception}
 *
 *  Potential Error Codes:
 *		TBD
 */
struct PriceCheckResponse
{
	/**
	 *	Reason for inability to provide final price, if any.
	 *
	 *	When the final price is provided, this will be set to SUCCESS.
	 */
	1:required PriceCheckReason reason,
	
	/**
	 *	Hint to app as to best way to obtain next value needed.
	 *
	 *	Primarily used for VIN retrieval to bypass barcode and OCR when we know it won't work.
	 */
	2:optional PriceCheckHint hint,
	
	/**
	 *	Result of Price Check.
	 *
	 *	This is guaranteed to be set when the reason is SUCCESS, but it may be set prior to this. For example, when a
	 *	list of VINs is provided in the request, and the server has chosen one from the list, the chosen VIN will be in
	 *	this result. The app should take any values provided in the result and place them into the next request to
	 *	"lock" them in and reduce the amount of work the server will do on the next pass.
	 */
	3:optional PriceCheckResult result,
	
	/**
	 *	Set of possible values to present to user.
	 *
	 *	For many of the possible non-SUCCESS reasons, a set of values for the user to choose from is provided here. This
	 *	may be dealers, makes, years, models, trims, packages, options, or incentives.
	 */
	4:optional list<PriceCheckPossibleValueGroup> possibleValueGroups,
}
