/*
 * This file provides definitions for user-related operations.
 */
 
include "TCThriftContact.thrift"

namespace java com.truecar.thrift.user
namespace cocoa TCThrift

/**
 * Enumeration of recognized alternative authentication providers:
 *
 * <dl>
 * <dt>GOOGLE</dt>
 * <dd>OAuth authenication via Google.</dd>
 * <dt>FACEBOOK</dt>
 * <dd>OAuth authentication via Facebook.</dd>
 * <dt>YAHOO</dt>
 * <dd>OAuth authentication via Yahoo.</dd>
 * </dl>
 */
enum UserAlternativeAuth {
	GOOGLE,
	FACEBOOK,
	YAHOO
}

/**
 * A verification token. This information is normally delivered to the user
 * via a verification email.
 */
struct VerificationToken {
	/**
	 * The value of the verification token. This is opaque information in
	 * an encoded form.
	 */
	1:required string value,

	/**
	 * This is the token timestamp in an encoded form.
	 */
	2:required string timestamp
}

/**
 * This union enumerates the types of user credentials accepted for various
 * authentication requests.
 */
struct UserCredential {
	/**
	 * A password is only necessary if it has already been established in the
	 * user's account.
	 */
	1:optional string password,

	/**
	 * A verification token, received via email, can be used as a credential
	 * in some cases.
	 */
	2:optional VerificationToken verificationToken
}

/**
 * The result data for a successful user registration.
 */
struct UserResult {
	/** An opaque identifier for a unique user identity. */
	1:required string userHandle,

	/** A secure token for subsequent operations. */
	2:required string secureToken,

	/**
	 * Current contact information for the user. If the request updated this
	 * information, the result contains the merged result of that update and
	 * any existing information. If it was not updated, this represents the
	 * current contact information for the user.
	*/
	3:optional TCThriftContact.Contact contact,
	
	/**
	 * Flag to indicate if server has valid contact information, even if it
	 * was unable to return it due to security restrictions. When true, leads
	 * may be submitted without requiring contact information to be set.
	 */
	4:required bool isContactValid;
}

/**
 * Result data returned when authentication fails due to a missing
 * verification token. This can occur if no verification token is supplied in
 * the request, and the user's account has no password associated with it.
 *
 * This structure is deliberately empty.
 */
struct UserVerificationRequired {
}

/**
 * Result data returned when authentication fails due to a missing password.
 * This can occur if no password is supplied in the request, but the user's
 * account has a password associated with it.
 */
struct UserPasswordRequired {
	/**
	 * A URL that references the password reset page on a TrueCar web site.
	 */
	1:optional string resetUrl
}

/**
 * This response is returned if an existing user originally registered via
 * an OAuth provider. The implication is that this method must be used to
 * perform a sucessful authentication.
 */
struct UserAlternativeAuthRequired {
	/**
	 * This is an identifier for the source of the external authentication.
	 */
	1:required UserAlternativeAuth authenticator,

	/**
	 * This is the URL for a login screen. The implication is that the user
	 * could use that login to establish a password, which could then be used
	 * for normal authentication.
	*/
	2:optional string loginUrl
}
