include "TCThriftContact.thrift"
include "TCThriftIncentive.thrift"
include "ConsumerApp/PriceExperience.thrift"
include "TCThriftUser.thrift"

namespace cocoa TCThrift
namespace java com.truecar.mobile.thriftApi

//
//	BaseURL
//
//	The base URL for this API is https://{host}/api/{version}.
//	The base path is thus /api/{version}.
//

//
//	ApplicationUpdate API Endpoint
//
//	This API endpoint is used to determine if the application is up-to-date. When an update is needed, this API endpoint
//	will return a URL that can be used to retrieve the update. This API is called when the application is activated. If
//	the request fails do to lack of connectivity, it will be retried periodically while the application is running until
//	the application is deactivated or a response is retrieved. The last response from the server is stored locally to
//	ensure that the user cannot bypass a mandatory update even when there is no connectivity.
//
//	URL: {baseUrl}/update/app/{appId}/version/{appVersion}
//
//	Request:
//		GET {basePath}/update/app/{appId}/version/{appVersion} HTTP/1.1
//		Host: {host}
//		Accept: application/x-thrift
//
//	Response (success):
//		HTTP/1.1 200 OK
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{ApplicationUpdateResponse}
//
//	Response (error):
//		HTTP/1.1 {non200Status}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{Exception}
//
//	Potential Error Codes:
//		AppIdInvalid
//		AppVersionInvalid
//		AppNotFound
//		AppVersionNotFound
//
struct ApplicationUpdateResponse {
	1:optional string appUpdateUrl,		//	itms URL in production to go to App Store; empty if no update available
	2:optional bool mandatoryUpdate,	//	true if app must be updated to continue
	3:optional string desc,
}


//	--------------------------------------------------------------------------------------------------------------------

//
//	AuthenticateUser API Endpoint
//
//	This API endpoint is used to login with a username and password. The API returns a secure token which must be passed
//	in the X-TrueCar-Token HTTP header on later requests that deal with user-specific data. Additionally, in the spirit
//	of fat APIs, the response also returns the contact information for the user as currently known by the server.
//
//	URL: {baseUrl}/authenticate
//
//	Request:
//		POST {basePath}/authenticate HTTP/1.1
//		Host: {host}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//		Accept: application/x-thrift
//
//		{AuthenticateUserRequest}
//
//	Response (success):
//		HTTP/1.1 200 OK
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{AuthenticateUserResponse}
//
//	Response (error):
//		HTTP/1.1 {non200Status}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{Exception}
//
//	Potential Error Codes:
//		UserNameInvalid
//		PasswordInvalid
//		PagIdInvalid
//		PagIdNotFound
//		AuthenticationFailed - Mismatched user name / password combination with no indication as to which is incorrect.
//
//	Notes:
//		To thwart potential attacks, this API endpoint should have rate-limiting behavior attached to prevent fast
//		repeated calls from the same IP address.
//
struct AuthenticateUserRequest
{
	1:required string userEmail,
	2:required i64 pagId,
	3:optional TCThriftUser.UserCredential credential,
}

struct AuthenticateUserResponse
{
	/** The result of a successful operation. */
	1:optional TCThriftUser.UserResult success,

	/**
	 * The account is password-protected, but no password was supplied in the
	 * request.
	*/
	2:optional TCThriftUser.UserVerificationRequired verificationRequired,

	/**
	 * The account is password-protected, but no password was supplied in the
	 * request.
	*/
	3:optional TCThriftUser.UserPasswordRequired passwordRequired,

	/**
	 * Authentication was originally supplied via an external provider, which
	 * must now be used for subsequent authentication requests.
	*/
	4:optional TCThriftUser.UserAlternativeAuthRequired altAuthRequired
}

//	--------------------------------------------------------------------------------------------------------------------

//
//	UpdateContact API Endpoint
//
//	The Update Contact API is used to update the contact information for the user. This API has no synchronization
//	aspects as the user will be waiting for the server.
//
//	URL: {baseUrl}/contact/user/{userHandle}
//
//	Request:
//		POST {basePath}/contact/user/{userHandle} HTTP/1.1
//		Host: {host}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//		Accept: application/x-thrift
//		X-TrueCar-Token: {secureToken}
//
//		{UpdateContactRequest}
//
//	Response (success):
//		HTTP/1.1 200 OK
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{UpdateContactResponse}
//
//	Response (error):
//		HTTP/1.1 {non200Status}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{Exception}
//
//	Potential Error Codes:
//		TokenInvalid
//		TokenExpired
//		UserHandleInvalid
//		UserNotFound
//		FirstNameInvalid
//		LastNameInvalid
//		AddressInvalid
//		CityInvalid
//		StateInvalid
//		PostalCodeInvalid
//		PhoneInvalid
//		EmailInvalid
//		SearchPostalCodeInvalid
//
struct UpdateContactRequest
{
	1:required TCThriftContact.Contact contact,
}

struct UpdateContactResponse
{
	1:optional TCThriftContact.Contact contact,
	2:optional PriceExperience.PriceExperience defaultPriceExperience = PriceExperience.PriceExperience.RESTRICTED,

	/**
	 * Flag to indicate if server has valid contact information, even if it
	 * was unable to return it due to security restrictions. When true, leads
	 * may be submitted without requiring contact information to be set.
	 */
	3:required bool isContactValid;
}


//	--------------------------------------------------------------------------------------------------------------------

//
//	SetMemberPassword API Endpoint
//
//	This API endpoint is used to a password for a member.  Passwords must be at least 6 characters and contain at least one letter and one number.
//
//	URL: {baseUrl}/authenticate/pag/{pag}/user/{userHandle}/setpassword
//
//	Request:
//		POST {basePath}/authenticate/pag/{pag}/user/{userHandle}/setpassword HTTP/1.1
//		Host: {host}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//		Accept: application/x-thrift
//		X-TrueCar-Token: {secureToken}
//
//		{AuthenticateUserRequest}
//
//	Response (success):
//		HTTP/1.1 200 OK
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//
//	Response (error):
//		HTTP/1.1 {non200Status}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{Exception}
//
//	Potential Error Codes:
//		UserNameInvalid
//		PasswordInvalid
//		PasswordNotAllowed
//		PagIdInvalid
//		PagIdNotFound
//		AuthenticationFailed - Mismatched user name / password combination with no indication as to which is incorrect.
//		TokenInvalid
//		TokenExpired
//
//
struct SetMemberPasswordRequest
{
    1:required string password,
}



//	--------------------------------------------------------------------------------------------------------------------

//
//	GetIncentives API Endpoint
//
//	This API endpoint is used to retrieve the incentives for a trim.
//
//	URL: {baseUrl}/incentives/pag/{pag}/trimId/{trimId}/postalCode/{postalCode}
//
//	Request:
//		GET {baseUrl}/incentives/pag/{pag}/trimId/{trimId}/postalCode/{postalCode} HTTP/1.1
//		Host: {host}
//		Accept: application/x-thrift
//
//	Response (success):
//		HTTP/1.1 200 OK
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{GetIncentivesResponse}
//
//	Response (error):
//		HTTP/1.1 {non200Status}
//		Content-Type: application/x-thrift
//		Content-Length: {length}
//
//		{Exception}
//
//  Potential Error Codes:
//		TrimIdInvalid
//		TrimIdNotFound
//		
struct GetIncentivesResponse
{
	1:required list<TCThriftIncentive.Incentive> incentives,
}
